import arcpy,os,sys,winsound


LYR_LATS=arcpy.GetParameterAsText(0) #User input from toolbox dialogue box "LATERAL LINE layer"
LYR_CONPTS=arcpy.GetParameterAsText(1) #User input from toolbox dialogue box "CONNECTION POINT layerr"
LYR_MAINS=arcpy.GetParameterAsText(2) #MAINS input
alert = arcpy.GetParameter(3) #alert
UserSearchDist = arcpy.GetParameterAsText(4) #take as input


#SAFETY NET:
rowAmount = arcpy.GetCount_management(LYR_CONPTS) #get amount of rows
rowAmountInt = (int(rowAmount[0])) #convert to an int, for if statement comparison

print(rowAmountInt)
arcpy.AddMessage("Number of rows being processed:{}".format(rowAmountInt))

if rowAmountInt > 250: #We want to cancel the script if there are too many rows, this implies wrong dataset has been selected to process. We only want to process small sub-sections. not whole database. 
	arcpy.AddMessage("QUITTING SCRIPT!, too many points selected. (Check you are not processing main dataset)")
	sys.exit()



FLYR_Lats=arcpy.MakeFeatureLayer_management(LYR_LATS) #creates layer out of getPerameterAsText(0) - Requires connection point -  LAT LINE LAYER
FLYR_ConPts=arcpy.MakeFeatureLayer_management(LYR_CONPTS) #creates layer out of getPerameterAsText(1) - Requires lateral line - POINT LAYER
FLYR_Mains=arcpy.MakeFeatureLayer_management(LYR_MAINS)
#FLYR_Centroids=arcpy.MakeFeatureLayer_management(LYR_CENTROID)

SOURCEPATH=arcpy.Describe(FLYR_ConPts).Path.split("\\")
WORKSPACE=SOURCEPATH[0]+"\\"+SOURCEPATH[1]

print(WORKSPACE)

aExclude=[]
loopCount = 0


arcpy.env.overwriteOutput=True #allows overide of already created files.

#edit = arcpy.da.Editor(WORKSPACE)
#edit.startEditing(False, True)
#edit.startOperation()


# Load the Analysis toolbox so that the Near tool is available
arcpy.toolbox = "analysis"





def calculateDistance(x1,y1,x2,y2):

     dist = math.sqrt((x2 - x1)**2 + (y2 - y1)**2)
     return dist





accPtFilter = "NOT ((CaptMethod = 'As Constructed' And HzAccuracy IN ('0-50mm', '50-300mm_H', '50-500mm_V', '50-500mm')) Or ((Source like 'As_Con%' or source like 'As constructed data' or source like 'As constructed plan') And CaptMethod = 'Unknown' And HzAccuracy IN ('0-50mm', '50-300mm_H', '50-500mm_V', '50-500mm')) Or (CaptMethod = 'Surveyed / GPS' And HzAccuracy = '0-50mm'))"


searchDists = UserSearchDist.split(',')
arcpy.AddMessage("Using search distance(s): {}".format(searchDists))


with arcpy.da.UpdateCursor(FLYR_ConPts,"SHAPE@",accPtFilter) as pt_UdCursor: #looping through points


	for oConPt in pt_UdCursor:
		loopCount += 1
		arcpy.AddMessage("--------Processing point {} of {}--------".format(loopCount,rowAmountInt))
	

		arcpy.SelectLayerByLocation_management(FLYR_Lats, "WITHIN_A_DISTANCE", oConPt, 0.1, "NEW_SELECTION")  #checking and SKIPPING any lat which already has conPt (Pts which were already correct before processing). 
		LatWPtCount = int(arcpy.GetCount_management(FLYR_Lats).getOutput(0))
		if LatWPtCount > 0:
			arcpy.AddMessage("Lateral already has a point, Skipping!!")
			#arcpy.SelectLayerByAttribute_management(FLYR_Lats, "CLEAR_SELECTION")
			continue #breaks oConPt loop to skip completed point. Process next point.		


		#CODE TO FIND CLOSEST iteratively 
		aTested = []
		for iDist in searchDists:#,750,1000]:

			#Select nearby lateral to Points
			#SelectLayerByLocation_management (in_layer (selected from this layer), {overlap_type}, {select_features (evaluated against)}, {search_distance}, {selection_type}, {invert_spatial_relationship})
			
			sQuery="OBJECTID IN("             #exclusion list query
			sQuery=sQuery+"{})".format(",".join(aExclude))
			#arcpy.AddMessage("DEBUG:*sQuery contains: {}*".format(sQuery))
			
			arcpy.AddMessage("|--| SearchDist = {}m |--|".format(iDist))
			arcpy.SelectLayerByAttribute_management(FLYR_Lats, "CLEAR_SELECTION")
			#arcpy.AddMessage("Cleared selection. No of lats selected = {}".format(int(arcpy.GetCount_management(FLYR_Lats).getOutput(0))))
			remainingLats = arcpy.SelectLayerByLocation_management(FLYR_Lats, "WITHIN_A_DISTANCE", oConPt, iDist, "NEW_SELECTION") #selecting Lats within dist of ConPts.      #IF THIS IS NOT ASSIGNED A VARIABLE IT DOESNT WORK!
			#latCount = int(arcpy.GetCount_management(FLYR_Lats).getOutput(0))
			arcpy.AddMessage("No of lats within area: {}".format(int(arcpy.GetCount_management(FLYR_Lats).getOutput(0))))
			#arcpy.AddMessage("oCosestlat[0] :{}".format(FLYR_Lats))
			
			
			if len(aExclude) > 0:
				remainingLats = arcpy.SelectLayerByAttribute_management(FLYR_Lats, "REMOVE_FROM_SELECTION", sQuery) #remove exclusion list items from selection.  
				arcpy.AddMessage("No of lats within area after removal of excluded: {}".format(int(arcpy.GetCount_management(FLYR_Lats).getOutput(0))))
				
			
			iNumLats = int(arcpy.GetCount_management(remainingLats).getOutput(0))
			if iNumLats==0:
				arcpy.AddMessage( " 0 Laterals found within {0}m".format(iDist))
				continue #continue loop w/next iteration because nothing found.
			
			
			
			if iNumLats == 1:
				arcpy.AddMessage( "one Lateral found! at {0}m".format(iDist))
				
				
				# using selected lateral

				latCursor=arcpy.da.SearchCursor(remainingLats, ["SHAPE@","OID@"]) # OID@ USED TO EXCLUDE LATS FROM PROCESSING AFTER THEY've had a point connected to them 
				for lat in latCursor:
					#arcpy.AddMessage("In for lat in Latcursor loop!") #debug 
					
					#arcpy.AddMessage("ID of lat (In inumLats=1): {}".format(latCursor[1]))
					
					latFpt = arcpy.PointGeometry(arcpy.Point(lat[0].firstPoint.X, lat[0].firstPoint.Y)) #convert to new points
					latLpt = arcpy.PointGeometry(arcpy.Point(lat[0].lastPoint.X, lat[0].lastPoint.Y))
					
					checkDistFP = 0.4                                  
					arcpy.SelectLayerByLocation_management(FLYR_Mains, "INTERSECT",latLpt, checkDistFP, "NEW_SELECTION") #check if lat LP connected to main.  
					
					mainSelectedCount = int(arcpy.GetCount_management(FLYR_Mains).getOutput(0))                 
					
					if mainSelectedCount == 1:
						arcpy.AddMessage("LP is connected to main")
						#move point to FP
						oConPt[0]=latFpt
						pt_UdCursor.updateRow(oConPt)
						break
						
					else:						
						arcpy.AddMessage("FP is connected to main")
						#move point to LP
						oConPt[0]=latLpt
						pt_UdCursor.updateRow(oConPt)
						break						
													
				#arcpy.AddMessage("OID for lat IN SINGLE LAT = {}".format(lat[1]))
				aExclude.append(str(lat[1]))
				break #breaking idist loop because point moved to lat. 
								
				
				
			if iNumLats > 1:
				arcpy.AddMessage("More than one lateral found within point buffer of {}, skipping or calcing dist...".format(iDist))
				

				if True == True: #if centroid (removed) == True
					arcpy.AddMessage( "Calculating closest lat to point")
					
					
					dist=10000.00
					point=None
					dPoints={} #set up list of distances of points
					fid = 0

					latCursor=arcpy.da.SearchCursor(remainingLats, ["SHAPE@","OID@"]) # OID@ USED TO EXCLUDE LATS FROM PROCESSING AFTER THEY've had a point connected to them 
					for lat in latCursor:
						
						latFpt = arcpy.PointGeometry(arcpy.Point(lat[0].firstPoint.X, lat[0].firstPoint.Y)) #convert to new points
						latLpt = arcpy.PointGeometry(arcpy.Point(lat[0].lastPoint.X, lat[0].lastPoint.Y))
						
						checkDistFP = 0.4                                  
						arcpy.SelectLayerByLocation_management(FLYR_Mains, "INTERSECT",latLpt, checkDistFP, "NEW_SELECTION") #check if lat FP connected to main. 
						#arcpy.AddMessage(FLYR_Mains) #DEBUG
						
						mainSelectedCount = int(arcpy.GetCount_management(FLYR_Mains).getOutput(0))
						#arcpy.AddMessage("mainSelectedCount contains: {} selected mains".format(mainSelectedCount)) # DEBUG
						
						if mainSelectedCount == 1:
							arcpy.AddMessage("LP is connected to main, calcing dist...")
							rowdist=calculateDistance(latFpt.firstPoint.X, latFpt.firstPoint.Y, oConPt[0].firstPoint.X, oConPt[0].firstPoint.Y) 
							
							if rowdist<dist:
								dist=rowdist							
							dPoints[rowdist]=latFpt							
							
							
						else:
							arcpy.AddMessage("FP is connected to main, calcing dist...")
							rowdist=calculateDistance(latLpt.firstPoint.X, latLpt.firstPoint.Y, oConPt[0].firstPoint.X, oConPt[0].firstPoint.Y) #calc dis between LP Lat and CustPt
							
							if rowdist<dist:
								dist=rowdist							
							dPoints[rowdist]=latLpt
							
														
						#arcpy.AddMessage("IOD for lat = {}".format(lat[1]))
						
					point=dPoints[dist]
					oConPt[0]=point
					pt_UdCursor.updateRow(oConPt)
					AlteredLat = arcpy.SelectLayerByLocation_management(remainingLats, "INTERSECT",oConPt, 0, "SUBSET_SELECTION") #Select the SINGLE lateral which had the point connected to it
					
					#safety check here to ensure have 1 pt selected
					CompletedLatCount = int(arcpy.GetCount_management(AlteredLat).getOutput(0))
					arcpy.AddMessage("CompletedLatCount = {}".format(CompletedLatCount))
					
					exLat = arcpy.da.SearchCursor(AlteredLat, ["SHAPE@","OID@"])
					for exclude in exLat:
						arcpy.AddMessage("exclude[1] = {}".format(exclude[1]))
						aExclude.append(str(exclude[1]))   #Oclosest LAT 1 is what? doesnt exist.     #exclude the lateral with the newly snapped point
						#arcpy.AddMessage("aExclude contains: {}".format(aExclude))
						
					break #breaking idist loop because Point moved to a lateral.
					
 		

#edit.stopOperation()
#edit.stopEditing(True)


if alert == True:
        arcpy.AddMessage("Sending audio alert...")
        winsound.Beep(250, 500)
        winsound.Beep(250, 500)




